@@ -68,6 +68,20 @@ class Session:
 
     notice_displayed = False
 
+    BAD_MSG_DESCRIPTION = {
+        16: "[16] msg_id too low, the client time has to be synchronized",
+        17: "[17] msg_id too high, the client time has to be synchronized",
+        18: "[18] incorrect two lower order msg_id bits, the server expects client message msg_id to be divisible by 4",
+        19: "[19] container msg_id is the same as msg_id of a previously received message",
+        20: "[20] message too old, it cannot be verified by the server",
+        32: "[32] msg_seqno too low",
+        33: "[33] msg_seqno too high",
+        34: "[34] an even msg_seqno expected, but odd received",
+        35: "[35] odd msg_seqno expected, but even received",
+        48: "[48] incorrect server salt",
+        64: "[64] invalid container"
+    }
+
     def __init__(self,
                  dc_id: int,
                  test_mode: bool,
@@ -89,9 +103,8 @@ class Session:
         self.auth_key = auth_key
         self.auth_key_id = sha1(auth_key).digest()[-8:]
 
-        self.msg_id = MsgId()
-        self.session_id = Long(self.msg_id())
-        self.msg_factory = MsgFactory(self.msg_id)
+        self.session_id = Long(MsgId())
+        self.msg_factory = MsgFactory()
 
         self.current_salt = None
 
@@ -146,7 +159,7 @@ class Session:
                 self.ping_thread.start()
 
                 log.info("Connection inited: Layer {}".format(layer))
-            except (OSError, TimeoutError):
+            except (OSError, TimeoutError, Error):
                 self.stop()
             else:
                 break
@@ -338,7 +351,10 @@ class Session:
         while True:
             packet = self.connection.recv()
 
-            if packet is None or (len(packet) == 4 and Int.read(BytesIO(packet)) == -404):
+            if packet is None or len(packet) == 4:
+                if packet:
+                    log.warning("Server sent \"{}\"".format(Int.read(BytesIO(packet))))
+
                 if self.is_connected.is_set():
                     Thread(target=self.restart, name="RestartThread").start()
                 break
@@ -370,6 +386,11 @@ class Session:
                 raise TimeoutError
             elif isinstance(result, types.RpcError):
                 Error.raise_it(result, type(data))
+            elif isinstance(result, types.BadMsgNotification):
+                raise Exception(self.BAD_MSG_DESCRIPTION.get(
+                    result.error_code,
+                    "Error code {}".format(result.error_code)
+                ))
             else:
                 return result
 
